#include "caosdb/filestreaming/FileWriter.h"
#include "caosdb/filestreaming/FileError.h" // for FileIOError, FileLockError
#include <boost/filesystem/path.hpp>        // for path
#include <mutex>                            // for try_to_lock
#include <utility>                          // for move

namespace FileExchange {

FileWriter::FileWriter(boost::filesystem::path filename)
  : filename_(std::move(filename)) {
  this->openFile();
}

FileWriter::FileWriter(boost::filesystem::path filename,
                       std::shared_ptr<FileMutex> mutexPtr)
  : filename_(std::move(filename)), mutexPtr_(std::move(mutexPtr)) {
  this->openFile();
}

void FileWriter::openFile() {
  if (mutexPtr_) {
    lock_ = FileWriteLock(*mutexPtr_, std::try_to_lock);
    if (!lock_) {
      throw FileLockError("Can't lock file for writing: " + filename_.string());
    }
  }

  stream_.open(filename_, std::ios::binary | std::ios::trunc);
  if (!stream_) {
    throw FileIOError("Can't open file for writing: " + filename_.string());
  }
}

void FileWriter::write(const std::string &buffer) {
  auto bufferSize = buffer.size();
  if (bufferSize > 0) {
    if (!stream_.write(buffer.data(), bufferSize)) {
      throw FileIOError("Can't write file: " + filename_.string());
    }
  }
}

} // namespace FileExchange
